@@ -5,14 +5,14 @@ import datetime
 
 from sql import Literal
 from sql.aggregate import Max, Sum
-from sql.functions import Extract, CharLength
+from sql.functions import Extract
 
 from trytond.i18n import gettext
-from trytond.model import ModelView, ModelSQL, fields, Unique
-from trytond.wizard import Wizard, StateView, StateAction, Button
-from trytond.pyson import Eval, PYSONEncoder, Date
-from trytond.transaction import Transaction
+from trytond.model import Index, ModelSQL, ModelView, Unique, fields
 from trytond.pool import Pool
+from trytond.pyson import Date, Eval, PYSONEncoder, TimeDelta
+from trytond.transaction import Transaction
+from trytond.wizard import Button, StateAction, StateView, Wizard
 
 from .exceptions import DurationValidationError
 
@@ -22,40 +22,44 @@ class Line(ModelSQL, ModelView):
     __name__ = 'timesheet.line'
     company = fields.Many2One('company.company', 'Company', required=True,
         help="The company on which the time is spent.")
-    employee = fields.Many2One('company.employee', 'Employee', required=True,
-        select=True, domain=[
+    employee = fields.Many2One(
+        'company.employee', "Employee", required=True, domain=[
             ('company', '=', Eval('company', -1)),
             ['OR',
                 ('start_date', '=', None),
-                ('start_date', '<=', Eval('date')),
+                ('start_date', '<=', Eval('date', None)),
                 ],
             ['OR',
                 ('end_date', '=', None),
-                ('end_date', '>=', Eval('date')),
+                ('end_date', '>=', Eval('date', None)),
                 ],
             ],
-        depends=['company', 'date'],
         help="The employee who spends the time.")
-    date = fields.Date('Date', required=True, select=True,
+    date = fields.Date(
+        "Date", required=True,
         help="When the time is spent.")
-    duration = fields.TimeDelta('Duration', 'company_work_time', required=True)
-    work = fields.Many2One('timesheet.work', 'Work',
-        required=True, select=True, domain=[
+    duration = fields.TimeDelta(
+        'Duration', 'company_work_time', required=True,
+        domain=[
+            ('duration', '>=', TimeDelta()),
+            ])
+    work = fields.Many2One(
+        'timesheet.work', "Work", required=True,
+        domain=[
             ('company', '=', Eval('company', -1)),
             ['OR',
                 ('timesheet_start_date', '=', None),
-                ('timesheet_start_date', '<=', Eval('date')),
+                ('timesheet_start_date', '<=', Eval('date', None)),
                 ],
             ['OR',
                 ('timesheet_end_date', '=', None),
-                ('timesheet_end_date', '>=', Eval('date')),
+                ('timesheet_end_date', '>=', Eval('date', None)),
                 ],
             ],
-        depends=['date', 'company'],
         help="The work on which the time is spent.")
     description = fields.Char('Description',
         help="Additional description of the work done.")
-    uuid = fields.Char("UUID", readonly=True)
+    uuid = fields.Char("UUID", readonly=True, strip=False)
 
     @classmethod
     def __setup__(cls):
@@ -69,40 +73,27 @@ class Line(ModelSQL, ModelView):
             ('uuid_unique', Unique(t, t.uuid),
                 'timesheet.msg_line_uuid_unique'),
             ]
-
-    @classmethod
-    def __register__(cls, module_name):
-        transaction = Transaction()
-        cursor = transaction.connection.cursor()
-        update = transaction.connection.cursor()
-        table = cls.__table_handler__(module_name)
-        sql_table = cls.__table__()
-        pool = Pool()
-        Work = pool.get('timesheet.work')
-        work = Work.__table__()
-
-        created_company = not table.column_exist('company')
-
-        super(Line, cls).__register__(module_name)
-
-        # Migration from 3.4: new company field
-        if created_company:
-            # Don't use FROM because SQLite nor MySQL support it.
-            update.execute(*sql_table.update(
-                    [sql_table.company], [work.select(work.company,
-                            where=work.id == sql_table.work)]))
-        # Migration from 3.4: change hours into timedelta duration
-        if table.column_exist('hours'):
-            table.drop_constraint('check_move_hours_pos')
-            cursor.execute(*sql_table.select(
-                    sql_table.id, sql_table.hours))
-            for id_, hours in cursor:
-                duration = datetime.timedelta(hours=hours)
-                update.execute(*sql_table.update(
-                        [sql_table.duration],
-                        [duration],
-                        where=sql_table.id == id_))
-            table.drop_column('hours')
+        cls._sql_indexes.update({
+                Index(
+                    t,
+                    (t.date, Index.Equality()),
+                    (t.company, Index.Equality()),
+                    (t.employee, Index.Equality())),
+                Index(
+                    t,
+                    (t.date, Index.Range()),
+                    (t.employee, Index.Equality())),
+                Index(
+                    t,
+                    (Extract('YEAR', t.date), Index.Equality()),
+                    (Extract('WEEK', t.date), Index.Equality()),
+                    (t.employee, Index.Equality())),
+                Index(
+                    t,
+                    (Extract('YEAR', t.date), Index.Equality()),
+                    (Extract('MONTH', t.date), Index.Equality()),
+                    (t.employee, Index.Equality())),
+                })
 
     @staticmethod
     def default_company():
@@ -127,16 +118,19 @@ class Line(ModelSQL, ModelView):
         return Transaction().context.get('date') or Date_.today()
 
     @classmethod
-    def validate(cls, lines):
-        super(Line, cls).validate(lines)
-        for line in lines:
-            line.check_duration()
+    def validate_fields(cls, lines, field_names):
+        super().validate_fields(lines, field_names)
+        cls.check_duration(lines, field_names)
 
-    def check_duration(self):
-        if self.duration < datetime.timedelta():
-            raise DurationValidationError(
-                gettext('timesheet.msg_line_duration_positive',
-                    line=self.rec_name))
+    @classmethod
+    def check_duration(cls, lines, field_names=None):
+        if field_names and 'duration' not in field_names:
+            return
+        for line in lines:
+            if line.duration < datetime.timedelta():
+                raise DurationValidationError(
+                    gettext('timesheet.msg_line_duration_positive',
+                        line=line.rec_name))
 
     @classmethod
     def copy(cls, lines, default=None):
@@ -170,14 +164,13 @@ class EnterLinesStart(ModelView):
             ('company', '=', Eval('context', {}).get('company', -1)),
             ['OR',
                 ('start_date', '=', None),
-                ('start_date', '<=', Eval('date')),
+                ('start_date', '<=', Eval('date', None)),
                 ],
             ['OR',
                 ('end_date', '=', None),
-                ('end_date', '>=', Eval('date')),
+                ('end_date', '>=', Eval('date', None)),
                 ],
             ],
-        depends=['date'],
         help="The employee who spends the time.")
     date = fields.Date('Date', required=True,
         help="When the time is spent.")
@@ -266,10 +259,8 @@ class HoursEmployeeContext(ModelView):
 class HoursEmployeeWeekly(ModelSQL, ModelView):
     'Hours per Employee per Week'
     __name__ = 'timesheet.hours_employee_weekly'
-    year = fields.Char('Year')
-    week_internal = fields.Char('Week')
-    week = fields.Function(fields.Char('Week'),
-        'get_week', searcher='search_week')
+    year = fields.Integer("Year")
+    week = fields.Integer("Week")
     employee = fields.Many2One('company.employee', 'Employee')
     duration = fields.TimeDelta('Duration', 'company_work_time')
 
@@ -285,11 +276,8 @@ class HoursEmployeeWeekly(ModelSQL, ModelView):
         pool = Pool()
         Line = pool.get('timesheet.line')
         line = Line.__table__()
-        type_name = cls.year.sql_type().base
-        year_column = Extract('YEAR', line.date).cast(type_name).as_('year')
-        type_name = cls.week_internal.sql_type().base
-        week_column = Extract('WEEK', line.date).cast(type_name).as_(
-            'week_internal')
+        year_column = Extract('YEAR', line.date).as_('year')
+        week_column = Extract('WEEK', line.date).as_('week')
         return line.select(
             Max(Extract('WEEK', line.date)
                 + Extract('YEAR', line.date) * 100
@@ -304,23 +292,11 @@ class HoursEmployeeWeekly(ModelSQL, ModelView):
             Sum(line.duration).as_('duration'),
             group_by=(year_column, week_column, line.employee))
 
-    def get_week(self, name):
-        return '%02i' % int(self.week_internal)
-
-    @classmethod
-    def search_week(cls, name, domain):
-        return [('week_internal',) + tuple(domain[1:])]
-
-    @classmethod
-    def order_week(cls, tables):
-        table, _ = tables[None]
-        return [CharLength(table.week_internal), table.week_internal]
-
 
 class HoursEmployeeMonthly(ModelSQL, ModelView):
     'Hours per Employee per Month'
     __name__ = 'timesheet.hours_employee_monthly'
-    year = fields.Char('Year')
+    year = fields.Integer("Year")
     month = fields.Many2One('ir.calendar.month', "Month")
     employee = fields.Many2One('company.employee', 'Employee')
     duration = fields.TimeDelta('Duration', 'company_work_time')
@@ -339,8 +315,7 @@ class HoursEmployeeMonthly(ModelSQL, ModelView):
         Month = pool.get('ir.calendar.month')
         line = Line.__table__()
         month = Month.__table__()
-        type_name = cls.year.sql_type().base
-        year_column = Extract('YEAR', line.date).cast(type_name).as_('year')
+        year_column = Extract('YEAR', line.date).as_('year')
         month_index = Extract('MONTH', line.date)
         return line.join(month, condition=month_index == month.id).select(
             Max(Extract('MONTH', line.date)
