@@ -2,13 +2,11 @@
 # this repository contains the full copyright notices and license terms.
 import datetime
 
-from werkzeug.exceptions import abort
-from werkzeug.wrappers import Response
-
+from trytond.protocols.wrappers import (
+    Response, abort, allow_null_origin, user_application, with_pool,
+    with_transaction)
 from trytond.transaction import Transaction
 from trytond.wsgi import app
-from trytond.protocols.wrappers import (
-    with_pool, with_transaction, user_application, allow_null_origin)
 
 timesheet_application = user_application('timesheet')
 
