@@ -7,11 +7,11 @@ from sql import Literal
 from sql.aggregate import Sum
 
 from trytond.i18n import gettext
-from trytond.model import ModelView, ModelSQL, ModelStorage, fields, Unique
-from trytond.pyson import Bool, Eval, If
-from trytond.transaction import Transaction
+from trytond.model import ModelSQL, ModelStorage, ModelView, Unique, fields
 from trytond.pool import Pool
-from trytond.tools import reduce_ids, grouped_slice
+from trytond.pyson import Bool, Eval, If
+from trytond.tools import grouped_slice, reduce_ids
+from trytond.transaction import Transaction
 
 from .exceptions import CompanyValidationError
 
@@ -24,14 +24,12 @@ class Work(ModelSQL, ModelView):
             'invisible': Bool(Eval('origin')),
             'required': ~Eval('origin'),
             },
-        depends=['origin'],
         help="The main identifier of the work.")
     origin = fields.Reference('Origin', selection='get_origin',
         states={
             'invisible': Bool(Eval('name')),
             'required': ~Eval('name'),
             },
-        depends=['name'],
         help="Use to relate the time spent to other records.")
     duration = fields.Function(fields.TimeDelta('Timesheet Duration',
             'company_work_time', help="Total time spent on this work."),
@@ -42,7 +40,6 @@ class Work(ModelSQL, ModelView):
                 ('timesheet_start_date', '<=', Eval('timesheet_end_date')),
                 ()),
             ],
-        depends=['timesheet_end_date'],
         help="Restrict adding lines before the date.")
     timesheet_end_date = fields.Date('Timesheet End',
         domain=[
@@ -50,10 +47,10 @@ class Work(ModelSQL, ModelView):
                 ('timesheet_end_date', '>=', Eval('timesheet_start_date')),
                 ()),
             ],
-        depends=['timesheet_start_date'],
         help="Restrict adding lines after the date.")
-    company = fields.Many2One('company.company', 'Company', required=True,
-        select=True, help="Make the work belong to the company.")
+    company = fields.Many2One(
+        'company.company', "Company", required=True,
+        help="Make the work belong to the company.")
     timesheet_lines = fields.One2Many('timesheet.line', 'work',
         'Timesheet Lines',
         help="Spend time on this work.")
@@ -70,45 +67,6 @@ class Work(ModelSQL, ModelView):
                 'timesheet.msg_work_origin_unique_company'),
             ]
 
-    @classmethod
-    def __register__(cls, module_name):
-        table_h = cls.__table_handler__(module_name)
-        table = cls.__table__()
-        cursor = Transaction().connection.cursor()
-
-        super(Work, cls).__register__(module_name)
-
-        # Migration from 4.0: remove required on name
-        table_h.not_null_action('name', 'remove')
-
-        # Migration from 4.0: remove parent, left and right
-        if table_h.column_exist('parent'):
-            id2name = {}
-            id2parent = {}
-            cursor.execute(*table.select(
-                    table.id, table.parent, table.name))
-            for id_, parent, name in cursor:
-                id2name[id_] = name
-                id2parent[id_] = parent
-
-            for id_, name in id2name.items():
-                parent = id2parent[id_]
-                while parent:
-                    name = '%s\\%s' % (id2name[parent], name)
-                    parent = id2parent[parent]
-                cursor.execute(*table.update(
-                        [table.name], [name],
-                        where=table.id == id_))
-            table_h.drop_column('parent')
-        table_h.drop_column('left')
-        table_h.drop_column('right')
-
-        # Migration from 4.0: remove timesheet_available
-        if table_h.column_exist('timesheet_available'):
-            cursor.execute(*table.delete(
-                    where=table.timesheet_available == Literal(False)))
-            table_h.drop_column('timesheet_available')
-
     @staticmethod
     def default_company():
         return Transaction().context.get('company')
